==> Botan-1.1.8.tgz: Botan-1.1.8/src/asn1_dn.cpp <==
/*************************************************
* X509_DN Source File                            *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#include <botan/asn1_obj.h>
#include <botan/oids.h>

#include <iostream>

namespace Botan {

namespace {

/*************************************************
* Compare two RDN values for equality            *
*************************************************/
bool is_equal(const ASN1_String& attr1_str, const ASN1_String& attr2_str)
   {
   const std::string& attr1 = attr1_str.value();
   const std::string& attr2 = attr2_str.value();

   return (attr1 == attr2);
   }

}


/*************************************************
* Create an empty X509_DN                        *
*************************************************/
X509_DN::X509_DN()
   {
   }

/*************************************************
* Create an X509_DN                              *
*************************************************/
X509_DN::X509_DN(const std::map<OID, ASN1_String>& args)
   {
   dn_info = args;
   }

/*************************************************
* Create an X509_DN                              *
*************************************************/
X509_DN::X509_DN(const std::map<std::string, std::string>& args)
   {
   std::map<std::string, std::string>::const_iterator j;

   for(j = args.begin(); j != args.end(); j++)
      add_attribute(OIDS::lookup(j->first), j->second);
   }

/*************************************************
* Add an attribute to a X509_DN                  *
*************************************************/
void X509_DN::add_attribute(const std::string& type,
                            const ASN1_String& str)
   {
   OID oid = OIDS::lookup(type);
   add_attribute(oid, str);
   }

/*************************************************
* Add an attribute to a X509_DN                  *
*************************************************/
void X509_DN::add_attribute(const OID& oid, const ASN1_String& str)
   {
   if(str.value() == "")
      return;
   dn_info[oid] = str;
   }

/*************************************************
* Look up an attribute in the X509_DN            *
*************************************************/
std::string X509_DN::do_lookup(const OID& oid) const
   {
   std::map<OID, ASN1_String>::const_iterator iter = dn_info.find(oid);
   if(iter == dn_info.end())
      return "";
   return iter->second.value();
   }

/*************************************************
* Return the common name                         *
*************************************************/
std::string X509_DN::common_name() const
   {
   return do_lookup(OIDS::lookup("X500.CommonName"));
   }

/*************************************************
* Return the country code                        *
*************************************************/
std::string X509_DN::country_code() const
   {
   return do_lookup(OIDS::lookup("X500.Country"));
   }

/*************************************************
* Return the organization name                   *
*************************************************/
std::string X509_DN::organization() const
   {
   return do_lookup(OIDS::lookup("X500.Organization"));
   }

/*************************************************
* Return the organizational unit name            *
*************************************************/
std::string X509_DN::organizational_unit() const
   {
   return do_lookup(OIDS::lookup("X500.OrganizationalUnit"));
   }

/*************************************************
* Return the locality name                       *
*************************************************/
std::string X509_DN::locality() const
   {
   return do_lookup(OIDS::lookup("X500.Locality"));
   }

/*************************************************
* Return the state/province name                 *
*************************************************/
std::string X509_DN::state() const
   {
   return do_lookup(OIDS::lookup("X500.State"));
   }

/*************************************************
* Return the PKCS #9 email address               *
*************************************************/
std::string X509_DN::email() const
   {
   return do_lookup(OIDS::lookup("PKCS9.EmailAddress"));
   }

/*************************************************
* Get the attributes of this X509_DN             *
*************************************************/
std::map<OID, ASN1_String> X509_DN::get_attributes() const
   {
   return dn_info;
   }

/*************************************************
* Compare two X509_DNs for equality              *
*************************************************/
bool operator==(const X509_DN& dn1, const X509_DN& dn2)
   {
   std::map<OID, ASN1_String> attr1 = dn1.get_attributes();
   std::map<OID, ASN1_String> attr2 = dn2.get_attributes();

   typedef std::map<OID, ASN1_String>::const_iterator rdn_iter;

   if(attr1.size() != attr2.size())
      return false;

   for(rdn_iter j = attr1.begin(); j != attr1.end(); j++)
      {
      const OID& oid = j->first;
      if(attr2.find(oid) == attr2.end())
         return false;
      }

   for(rdn_iter j = attr1.begin(); j != attr1.end(); j++)
      {
      rdn_iter in_dn2 = attr2.find(j->first);

      if(in_dn2 == attr2.end())
         throw Internal_Error("X509_DN: Comparison internal failure");

      const ASN1_String& attr1_str = j->second;
      const ASN1_String& attr2_str = in_dn2->second;

      if(!is_equal(attr1_str, attr2_str))
         return false;
      }

   return true;
   }

/*************************************************
* Compare two X509_DNs for inequality            *
*************************************************/
bool operator!=(const X509_DN& dn1, const X509_DN& dn2)
   {
   return !(dn1 == dn2);
   }

namespace DER {

namespace {

/*************************************************
* DER encode a RelativeDistinguishedName         *
*************************************************/
void do_oid(DER_Encoder& encoder, std::map<OID, ASN1_String>& dn_info,
            const std::string& oid_str, bool must_exist)
   {
   const OID oid = OIDS::lookup(oid_str);
   const bool exists = (dn_info.find(oid) != dn_info.end());

   if(!exists && must_exist)
      throw Encoding_Error("X509_DN: No entry for " + oid_str);
   if(!exists) return;

   encoder.start_set();
   encoder.start_sequence();
   DER::encode(encoder, oid);
   DER::encode(encoder, dn_info[oid]);
   encoder.end_sequence();
   encoder.end_set();
   }

}

/*************************************************
* DER encode a DistinguishedName                 *
*************************************************/
void encode(DER_Encoder& encoder, const X509_DN& dn)
   {
   std::map<OID, ASN1_String> dn_info = dn.get_attributes();

   encoder.start_sequence();
   do_oid(encoder, dn_info, "X500.Country", true);
   do_oid(encoder, dn_info, "X500.State", false);
   do_oid(encoder, dn_info, "X500.Locality", false);
   do_oid(encoder, dn_info, "X500.Organization", false);
   do_oid(encoder, dn_info, "X500.OrganizationalUnit", false);
   do_oid(encoder, dn_info, "X500.CommonName", true);
   do_oid(encoder, dn_info, "PKCS9.EmailAddress", false);
   encoder.end_sequence();
   }

}

namespace BER {

/*************************************************
* Decode a BER encoded DistinguishedName         *
*************************************************/
void decode(BER_Decoder& source, X509_DN& dn)
   {
   std::map<OID, ASN1_String> dn_info;

   BER_Decoder sequence = BER::get_subsequence(source);

   while(sequence.more_items())
      {
      BER_Decoder rdn = BER::get_subset(sequence);
      while(rdn.more_items())
         {
         OID oid;
         ASN1_String value;

         BER_Decoder ava = BER::get_subsequence(rdn);
         BER::decode(ava, oid);
         BER::decode(ava, value);
         ava.verify_end();

         if(dn_info.find(oid) != dn_info.end())
            {
            if(oid == OIDS::lookup("X500.OrganizationalUnit"))
               dn_info[oid] = dn_info[oid].value() + " / " + value.value();
            else
               throw Decoding_Error("X509_DN: Repeated element " +
                                    oid.as_string());
            }

         dn_info[oid] = value;
         }
      }
   dn = X509_DN(dn_info);
   }

}

}

==> Botan-1.1.9.tgz: Botan-1.1.9/src/asn1_dn.cpp <==
/*************************************************
* X509_DN Source File                            *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#include <botan/asn1_obj.h>
#include <botan/oids.h>

namespace Botan {

/*************************************************
* Create an empty X509_DN                        *
*************************************************/
X509_DN::X509_DN()
   {
   }

/*************************************************
* Create an X509_DN                              *
*************************************************/
X509_DN::X509_DN(const std::map<OID, ASN1_String>& args)
   {
   dn_info = args;
   }

/*************************************************
* Create an X509_DN                              *
*************************************************/
X509_DN::X509_DN(const std::map<std::string, std::string>& args)
   {
   std::map<std::string, std::string>::const_iterator j;

   for(j = args.begin(); j != args.end(); j++)
      add_attribute(OIDS::lookup(j->first), j->second);
   }

/*************************************************
* Add an attribute to a X509_DN                  *
*************************************************/
void X509_DN::add_attribute(const std::string& type,
                            const ASN1_String& str)
   {
   OID oid = OIDS::lookup(type);
   add_attribute(oid, str);
   }

/*************************************************
* Add an attribute to a X509_DN                  *
*************************************************/
void X509_DN::add_attribute(const OID& oid, const ASN1_String& str)
   {
   if(str.value() == "")
      return;
   dn_info[oid] = str;
   }

/*************************************************
* Look up an attribute in the X509_DN            *
*************************************************/
std::string X509_DN::do_lookup(const OID& oid) const
   {
   std::map<OID, ASN1_String>::const_iterator iter = dn_info.find(oid);
   if(iter == dn_info.end())
      return "";
   return iter->second.value();
   }

/*************************************************
* Return the common name                         *
*************************************************/
std::string X509_DN::common_name() const
   {
   return do_lookup(OIDS::lookup("X500.CommonName"));
   }

/*************************************************
* Return the country code                        *
*************************************************/
std::string X509_DN::country_code() const
   {
   return do_lookup(OIDS::lookup("X500.Country"));
   }

/*************************************************
* Return the organization name                   *
*************************************************/
std::string X509_DN::organization() const
   {
   return do_lookup(OIDS::lookup("X500.Organization"));
   }

/*************************************************
* Return the organizational unit name            *
*************************************************/
std::string X509_DN::organizational_unit() const
   {
   return do_lookup(OIDS::lookup("X500.OrganizationalUnit"));
   }

/*************************************************
* Return the locality name                       *
*************************************************/
std::string X509_DN::locality() const
   {
   return do_lookup(OIDS::lookup("X500.Locality"));
   }

/*************************************************
* Return the state/province name                 *
*************************************************/
std::string X509_DN::state() const
   {
   return do_lookup(OIDS::lookup("X500.State"));
   }

/*************************************************
* Return the PKCS #9 email address               *
*************************************************/
std::string X509_DN::email() const
   {
   return do_lookup(OIDS::lookup("PKCS9.EmailAddress"));
   }

/*************************************************
* Get the attributes of this X509_DN             *
*************************************************/
std::map<OID, ASN1_String> X509_DN::get_attributes() const
   {
   return dn_info;
   }

/*************************************************
* Compare two X509_DNs for equality              *
*************************************************/
bool operator==(const X509_DN& dn1, const X509_DN& dn2)
   {
   std::map<OID, ASN1_String> attr1 = dn1.get_attributes();
   std::map<OID, ASN1_String> attr2 = dn2.get_attributes();

   typedef std::map<OID, ASN1_String>::const_iterator rdn_iter;

   if(attr1.size() != attr2.size())
      return false;

   for(rdn_iter j = attr1.begin(); j != attr1.end(); j++)
      {
      const OID& oid = j->first;
      if(attr2.find(oid) == attr2.end())
         return false;
      }

   for(rdn_iter j = attr1.begin(); j != attr1.end(); j++)
      {
      rdn_iter in_dn2 = attr2.find(j->first);

      if(in_dn2 == attr2.end())
         throw Internal_Error("X509_DN: Comparison internal failure");

      const std::string attr1_str = j->second.value();
      const std::string attr2_str = in_dn2->second.value();

      if(!x500_name_cmp(attr1_str, attr2_str))
         return false;
      }

   return true;
   }

/*************************************************
* Compare two X509_DNs for inequality            *
*************************************************/
bool operator!=(const X509_DN& dn1, const X509_DN& dn2)
   {
   return !(dn1 == dn2);
   }

namespace DER {

namespace {

/*************************************************
* DER encode a RelativeDistinguishedName         *
*************************************************/
void do_oid(DER_Encoder& encoder, std::map<OID, ASN1_String>& dn_info,
            const std::string& oid_str, bool must_exist)
   {
   const OID oid = OIDS::lookup(oid_str);
   const bool exists = (dn_info.find(oid) != dn_info.end());

   if(!exists && must_exist)
      throw Encoding_Error("X509_DN: No entry for " + oid_str);
   if(!exists) return;

   encoder.start_set();
   encoder.start_sequence();
   DER::encode(encoder, oid);
   DER::encode(encoder, dn_info[oid]);
   encoder.end_sequence();
   encoder.end_set();
   }

}

/*************************************************
* DER encode a DistinguishedName                 *
*************************************************/
void encode(DER_Encoder& encoder, const X509_DN& dn)
   {
   std::map<OID, ASN1_String> dn_info = dn.get_attributes();

   encoder.start_sequence();
   do_oid(encoder, dn_info, "X500.Country", true);
   do_oid(encoder, dn_info, "X500.State", false);
   do_oid(encoder, dn_info, "X500.Locality", false);
   do_oid(encoder, dn_info, "X500.Organization", false);
   do_oid(encoder, dn_info, "X500.OrganizationalUnit", false);
   do_oid(encoder, dn_info, "X500.CommonName", true);
   do_oid(encoder, dn_info, "PKCS9.EmailAddress", false);
   encoder.end_sequence();
   }

}

namespace BER {

/*************************************************
* Decode a BER encoded DistinguishedName         *
*************************************************/
void decode(BER_Decoder& source, X509_DN& dn)
   {
   std::map<OID, ASN1_String> dn_info;

   BER_Decoder sequence = BER::get_subsequence(source);

   while(sequence.more_items())
      {
      BER_Decoder rdn = BER::get_subset(sequence);
      while(rdn.more_items())
         {
         OID oid;
         ASN1_String value;

         BER_Decoder ava = BER::get_subsequence(rdn);
         BER::decode(ava, oid);
         BER::decode(ava, value);
         ava.verify_end();

         if(dn_info.find(oid) != dn_info.end())
            {
            if(oid == OIDS::lookup("X500.OrganizationalUnit"))
               dn_info[oid] = dn_info[oid].value() + " / " + value.value();
            else
               throw Decoding_Error("X509_DN: Repeated element " +
                                    oid.as_string());
            }

         dn_info[oid] = value;
         }
      }
   dn = X509_DN(dn_info);
   }

}

}
